@@ -12,7 +12,7 @@ cd build
 
 echo "[Drone build] cmake configuration"
 
-cmake -DDO_QUIET_MAKE=TRUE -DBOOST_ROOT=/usr ..
+cmake -DDO_QUIET_MAKE=TRUE -DBOOST_ROOT=/usr -DNO_IPO=TRUE -DCMAKE_BUILD_TYPE=RELEASE ..
 
 echo "[Drone build] making salmon and installing locally (this could take a while)"
 
