@@ -45,8 +45,14 @@ struct AlevinOpts {
   bool dumpCellEq;
   //Stop progress sumps
   bool quiet;
+  // just perform alignment and produce 
+  // an output directory with a RAD file. 
+  bool just_align;
+  bool sketch_mode;
   //flag for deduplication
   bool noDedup;
+  //flag for not performing external whitelisting
+  bool noWhitelist;
   //Number of generator threads
   uint32_t numParsingThreads;
   //Number of consumer threads
