@@ -3,7 +3,7 @@
 
 template <typename FragT> class NullFragmentFilter {
 public:
-  inline void processFrag(FragT* f) {}
+  inline void processFrag(FragT* /*f*/) {}
 };
 
 #endif // __NULL_FRAGMENT_FILTER_HPP__
