@@ -2,7 +2,8 @@
 #define READ_PAIR
 
 #include "LibraryFormat.hpp"
-#include "RapMapUtils.hpp"
+//#include "RapMapUtils.hpp"
+#include "pufferfish/Util.hpp"
 #include "SalmonMath.hpp"
 #include "SalmonUtils.hpp"
 #include "StadenUtils.hpp"
@@ -59,6 +60,8 @@ struct ReadPair {
 
   inline bam_seq_t* getRead1() { return read1; }
   inline bam_seq_t* getRead2() { return read2; }
+  inline int32_t readLen() { return bam_seq_len(getRead1()); }
+  inline int32_t mateLen() { return bam_seq_len(getRead2()); }
 
   inline LibraryFormat& libFormat() { return libFmt; }
   inline bool isPaired() const {
@@ -74,19 +77,40 @@ struct ReadPair {
     return (isPaired() or isLeftOrphan()) ? read1 : nullptr;
   }
 
-  inline rapmap::utils::MateStatus mateStatus() const {
+  inline pufferfish::util::MateStatus mateStatus() const {
     if (isPaired()) {
-      return rapmap::utils::MateStatus::PAIRED_END_PAIRED;
+      return pufferfish::util::MateStatus::PAIRED_END_PAIRED;
     } else if (isLeftOrphan()) {
-      return rapmap::utils::MateStatus::PAIRED_END_LEFT;
+      return pufferfish::util::MateStatus::PAIRED_END_LEFT;
     } else if (isRightOrphan()) {
-      return rapmap::utils::MateStatus::PAIRED_END_RIGHT;
+      return pufferfish::util::MateStatus::PAIRED_END_RIGHT;
     }
 
     std::cerr << "ReadPair.hpp : mateStatus() --- should not get here ";
     std::cerr << "this may be a bug.  Please report it\n";
 
-    return rapmap::utils::MateStatus::PAIRED_END_PAIRED;
+    return pufferfish::util::MateStatus::PAIRED_END_PAIRED;
+  }
+
+  inline bool haveASTag() const {
+    uint8_t* tp = reinterpret_cast<uint8_t*>(bam_aux_find(read1, "AS"));
+    return !(tp == NULL);
+  }
+
+  inline int32_t getAS() const {
+    int32_t s{0};
+    auto bf1 = bam_flag(read1);
+    auto bf2 = bam_flag(read2);
+    // if the read is mapped
+    if (!(bf1 & BAM_FUNMAP)) {
+      uint8_t* tl = reinterpret_cast<uint8_t*>(bam_aux_find(read1, "AS"));
+      s += (tl == NULL) ? 0 : bam_aux_i(tl); 
+    }
+    if (!(bf2 & BAM_FUNMAP)) {
+      uint8_t* tr = reinterpret_cast<uint8_t*>(bam_aux_find(read2, "AS"));
+      s += (tr == NULL) ? 0 : bam_aux_i(tr);
+    }
+    return s;
   }
 
   inline int32_t pos() const { return left(); }
