@@ -130,7 +130,12 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
                       std::atomic<size_t>& processedReads) {
 
   // Seed with a real random value, if available
-  std::random_device rd;
+  #if defined(__linux) && defined(__GLIBCXX__) && __GLIBCXX__ >= 20200128
+    std::random_device rd("/dev/urandom");
+  #else
+    std::random_device rd;
+  #endif  // defined(__GLIBCXX__) && __GLIBCXX__ >= 2020012
+
   auto& log = salmonOpts.jointLog;
 
   // Whether or not we are using "banking"
